@@ -4,6 +4,7 @@
 #include <sys/mount.h>
 #include <sys/wait.h>
 
+#include "int.h"
 #include "fdinfo.h"
 #include "autofs.h"
 #include "rst-malloc.h"
@@ -13,6 +14,7 @@
 #include "protobuf.h"
 #include "pipes.h"
 #include "crtools.h"
+#include "util.h"
 
 #include "images/autofs.pb-c.h"
 
@@ -288,17 +290,14 @@ static int autofs_revisit_options(struct mount_info *pm)
 	char *str;
 	int ret = -ENOMEM;
 
-	str = malloc(1024);
+	str = xmalloc(1024);
 	if (!str) {
-		pr_err("failed to allocate\n");
 		return -ENOMEM;
 	}
 
 	f = fopen_proc(getpid(), "mountinfo");
-	if (!f) {
-		pr_perror("Can't open %d mountinfo", getpid());
+	if (!f)
 		goto free_str;
-	}
 
 	while (fgets(str, 1024, f)) {
 		int mnt_id = -1;
@@ -570,7 +569,7 @@ static int autofs_dup_pipe(struct pstree_item *task,
 	struct pipe_info *pi = container_of(ple->desc, struct pipe_info, d);
 	unsigned flags = O_WRONLY;
 
-	new_fd = find_unused_fd(&rsti(task)->used, new_fd);
+	new_fd = find_unused_fd(task, new_fd);
 
 	if (dup_pipe_info(pi, flags, pi->d.ops) < 0) {
 		pr_err("Failed to dup pipe entry ID %#x PIPE_ID %#x\n",
@@ -580,12 +579,12 @@ static int autofs_dup_pipe(struct pstree_item *task,
 
 	if (dup_fle(task, ple, new_fd, flags) < 0) {
 		pr_err("Failed to add fd %d to process %d\n",
-				new_fd, task->pid.virt);
+				new_fd, vpid(task));
 		return -1;
 	}
 
 	pr_info("autofs: added pipe fd %d, flags %#x to %d\n",
-			new_fd, flags, task->pid.virt);
+			new_fd, flags, vpid(task));
 	return new_fd;
 }
 
@@ -829,17 +828,6 @@ static autofs_info_t *autofs_create_info(const struct mount_info *mi,
 	return i;
 }
 
-static struct fdinfo_list_entry *find_fle_by_fd(struct list_head *head, int fd)
-{
-	struct fdinfo_list_entry *fle;
-
-	list_for_each_entry(fle, head, ps_list) {
-		if (fle->fe->fd == fd)
-			return fle;
-	}
-	return NULL;
-}
-
 static struct fdinfo_list_entry *autofs_pipe_le(struct pstree_item *master,
 						AutofsEntry *entry)
 {
@@ -849,15 +837,15 @@ static struct fdinfo_list_entry *autofs_pipe_le(struct pstree_item *master,
 	if (entry->has_read_fd)
 		pipe_fd = entry->read_fd;
 
-	ple = find_fle_by_fd(&rsti(master)->fds, pipe_fd);
+	ple = find_used_fd(master, pipe_fd);
 	if (!ple) {
 		pr_err("Failed to find pipe fd %d in process %d\n",
-				pipe_fd, master->pid.virt);
+				pipe_fd, vpid(master));
 		return NULL;
 	}
 	if (ple->fe->type != FD_TYPES__PIPE) {
 		pr_err("Fd %d in process %d is not a pipe: %d\n", pipe_fd,
-				master->pid.virt, ple->fe->type);
+				vpid(master), ple->fe->type);
 		return NULL;
 	}
 	return ple;
@@ -874,13 +862,9 @@ static int autofs_create_fle(struct pstree_item *task, FdinfoEntry *fe,
 		return -1;
 	le = (void *)ALIGN((long)le, sizeof(int));
 
-	futex_init(&le->real_pid);
-	le->pid = task->pid.virt;
-	le->fe = fe;
+	fle_init(le, vpid(task), fe);
 
-	collect_gen_fd(le, rst_info);
-
-	collect_used_fd(le, rst_info);
+	collect_task_fd(le, rst_info);
 
 	list_add_tail(&le->desc_list, &desc->fd_info_head);
 	le->desc = desc;
@@ -890,6 +874,22 @@ static int autofs_create_fle(struct pstree_item *task, FdinfoEntry *fe,
 	return 0;
 }
 
+static int autofs_open_pipefd(struct file_desc *d, int *new_fd)
+{
+	struct fdinfo_list_entry *fle = file_master(d);
+	int ret;
+
+	if (fle->stage < FLE_OPEN) {
+		ret = open_pipe(d, new_fd);
+		if (ret != 0)
+			return ret;
+		set_fds_event(fle->pid);
+		return 1;
+	}
+
+	return autofs_post_open(d, fle->fe->fd);
+}
+
 static int autofs_create_pipe(struct pstree_item *task, autofs_info_t *i,
 			      struct fdinfo_list_entry *ple)
 {
@@ -900,13 +900,13 @@ static int autofs_create_pipe(struct pstree_item *task, autofs_info_t *i,
 	struct file_desc_ops *ops;
 	PipeEntry *pe;
 
-	fd = find_unused_fd(&rsti(task)->used, fd);
+	fd = find_unused_fd(task, fd);
 
 	ops = shmalloc(sizeof(*ops));
 	if (!ops)
 		return -1;
 	memcpy(ops, pi->d.ops, sizeof(*ops));
-	ops->post_open = autofs_post_open;
+	ops->open = autofs_open_pipefd;
 
 	pe = shmalloc(sizeof(*pe));
 	if (!pe)
@@ -956,7 +956,7 @@ static int autofs_add_mount_info(void *data)
 		entry->fd = autofs_dup_pipe(master, ple, entry->fd);
 		if (entry->fd < 0) {
 			pr_err("Failed to find free fd in process %d\n",
-					master->pid.virt);
+					vpid(master));
 			return -1;
 		}
 	}
