@@ -13,4 +13,10 @@ prep
 ./test/zdtm.py run -t zdtm/static/mntns_ghost --fault 2 --keep-going --report report || fail
 ./test/zdtm.py run -t zdtm/static/mntns_ghost --fault 4 --keep-going --report report || fail
 
+./test/zdtm.py run -t zdtm/static/mntns_ghost --fault 6 --report report || fail
+./test/zdtm.py run -t zdtm/static/mntns_link_remap --fault 6 --report report || fail
+./test/zdtm.py run -t zdtm/static/unlink_fstat03 --fault 6 --report report || fail
+
 ./test/zdtm.py run -t zdtm/static/env00 --fault 5 --keep-going --report report || fail
+./test/zdtm.py run -t zdtm/static/maps04 --fault 131 --keep-going --report report --pre 2:1 || fail
+./test/zdtm.py run -t zdtm/transition/maps008 --fault 131 --keep-going --report report --pre 2:1 || fail
